#include <avr/io.h>
#include <avr/interrupt.h>

#include <util/delay.h>

#include <stdlib.h>
#include <stdio.h>

#include "lcd.h"
#include "usart.h"
#include "util.h"


/* definitii note (in hertzi) */
#define c    261
#define d    294
#define e    329
#define f    349
#define g    391
#define gS   415
#define a    440
#define aS   455
#define b    466
#define cH   523
#define cSH  554
#define dH   587
#define dSH  622
#define eH   659
#define fH   698
#define fSH  740
#define gH   784
#define gSH  830
#define aH   880

uint16_t surprise_notes[] = {a, a, a, f, cH, a, f, cH, a, eH, eH, eH, fH, cH, gS, f, cH, a, aH, a, a, aH, gSH, gH, fSH, fH, fSH, aS, dSH, dH, cSH, cH, b, cH, f, gS, f, a, cH, a, cH, eH, aH, a, a, aH, gSH, gH, fSH, fH, fSH, aS, dSH, dH, cSH, cH, b, cH, f, gS, f, cH, a, f, cH, a};

uint16_t durations[] = {500, 500, 500, 350, 150, 500, 350, 150, 650, 500, 500, 500, 350, 150, 500, 350, 150, 650, 500, 300, 150, 400, 200, 200, 125, 125, 250, 250, 400, 200, 200, 125, 125, 250, 125, 500, 375, 125, 500, 375, 125, 650, 500, 300, 150, 400, 200, 200, 125, 125, 250, 250, 400, 200, 200, 125, 125, 250, 250, 500, 375, 125, 500, 375, 125, 650};

uint8_t num_notes = sizeof(durations) / sizeof(durations[0]);

ISR(TIMER1_OVF_vect)
{
    PORTD |= (1 << PD4);
}

ISR(TIMER1_COMPB_vect)
{
    PORTD &= ~(1 << PD4);
}

void IO_init()
{
    /* LED-ul RGB si speaker-ul sunt puse ca iesire */
    DDRB |= (1 << PB3);
    DDRD |= (1 << PD4) | (1 << PD5) | (1 << PD7);

    /* butoanele sunt puse ca intrare, cu rezistenta de pull-up */
    DDRB &= ~((1 << PB0) | (1 << PB1));
    PORTB |= ((1 << PB0) | (1 << PB1));

    /* intreruperile pentru butoane sunt activate */
    PCICR |= (1 << PCIE1);
    PCMSK1 |= (1 << PCINT8) | (1 << PCINT9);
}

void test_LEDs()
{
    /* functionarea LED-ului RGB este testata cat timp PB0 este apasat */
    while((PINB & (1 << PB0)) == 0)
    {
        PORTB |= (1 << PB3);
        _delay_ms(1000);
        PORTB &= ~(1 << PB3);

        PORTD |= (1 << PD4);
        _delay_ms(1000);
        PORTD &= ~(1 << PD4);

        PORTD |= (1 << PD7);
        _delay_ms(1000);
        PORTD &= ~(1 << PD7);

        _delay_ms(2000);
    }
}


ISR(PCINT1_vect)
{
    if((PINB & (1 << PB1)) == 0)
    {
        uint8_t crt_cs = (TCCR1B & (7 << CS10)) >> CS10;
        uint8_t new_cs = (crt_cs + 1) % 6;

        TCCR1B = (TCCR1B & ~(7 << CS10)) | (new_cs << CS10);

        _delay_ms(50);
    }

    if((PINB & (1 << PB0)) == 0)
    {
        OCR1B += ICR1 / 16;

        _delay_ms(50);
    }

}


void timer0_init()
{
    TCCR0A = (3 << WGM00) | (2 << COM0A0);
    TCCR0B = (1 << CS00);
    OCR0A = 0xC0;
}


void timer1_init_ex1()
{
    /* timer1 este configurat cu modul FastPWM, top ICR1 si prescaler 256 */
    TCCR1A |= (2 << WGM10);
    TCCR1B |= (4 << CS10) | (3 << WGM12);

    TIMSK1 |= (1 << OCIE1B) | (1 << TOIE1);

    /* Valoarea pentru care avem 1Hz */
    ICR1 = 62500;

    /* duty cycle-ul este setat initial la 25% */
    OCR1B = ICR1 / 4;
}


void timer1_init_ex2()
{
    /* timer1 este configurat cu modul FastPWM, top ICR1 si prescaler 1 */
    TCCR1A |= (2 << WGM10) | (2 << COM1B0);
    /* prescaler 1 pentru exercitiul 3. */
    TCCR1B |= (1 << CS10) | (3 << WGM12);

    /* Valoarea pentru care avem 1Hz */
    ICR1 = 62500;

    /* duty cycle-ul este setat initial la 25% */
    OCR1B = ICR1 / 4;
}


void timer1_init_ex6()
{
    /* timer1 este configurat cu modul CTC, top OCR1A si prescaler 1024 */
    TCCR1A |= (1 << COM1A0) | (1 << COM1B0);
    TCCR1B |= (5 << CS10) | (1 << WGM12);

    OCR1B = 0;
    OCR1A = 0;
}


void timer2_init()
{
    TCCR2A = (3 << WGM20) | (2 << COM2A0);
    TCCR2B = (1 << CS20);
    OCR2A = 0xC0;
}

void update_color()
{
    static uint8_t red_cycle = 100;
    static uint8_t blue_cycle = 0;
    static uint8_t green_cycle = 100;
    static uint8_t state = 0;

    switch (state)
    {
        case 0:
            red_cycle -= 10;
            blue_cycle += 10;
            if (red_cycle == 0 && green_cycle == 100 && blue_cycle == 100)
                state = 1;
            break;
        case 1:
            red_cycle += 10;
            green_cycle -= 10;
            if (red_cycle == 100 && blue_cycle == 100 && green_cycle == 0)
                state = 2;
            break;
        case 2:
            green_cycle += 10;
            blue_cycle -= 10;
            if (red_cycle == 100 && blue_cycle == 0 && green_cycle == 100)
                state = 0;
            break;
        default:
            break;
    }
    OCR0A = (uint8_t)((uint32_t)red_cycle * 0xFF / 100);
    OCR1B = (uint16_t)((uint32_t)blue_cycle * ICR1 / 100);
    OCR2A = (uint8_t)((uint32_t)green_cycle * 0xFF / 100);
}

void update_song()
{
    static uint8_t note_remaining = 1;
    static int8_t note_position = -1;

    if(--note_remaining == 0)
    {
        note_position = (note_position + 1) % num_notes;

        note_remaining = durations[note_position] / 25;
        uint16_t new_top = 16000000 / 1024 / 2 / surprise_notes[note_position];
	TCNT1 = 0;
        OCR1B = (uint32_t)new_top * OCR1B / OCR1A;
	OCR1A = new_top;
    }
}


int main(void)
{
    IO_init();
    USART0_init();
    LCD_init();
    LCD_generate_bars();

    test_LEDs();

    timer0_init();
    timer1_init_ex6();
    timer2_init();

    sei();

    uint8_t iterations = 0;
    USART0_print("Debug hello!");

    while(1)
    {
            iterations = (iterations + 1) % 4;
            if(iterations == 0)
                update_color();

            update_song();

            LCD_writeInstr(LCD_INSTR_clearDisplay);
            LCD_printAt(0, "R");
            PWM_draw_bar(OCR0A, 0xFF, 1, 4);
            LCD_printAt(5, "G");
            PWM_draw_bar(OCR2A, 0xFF, 6, 4);
            LCD_printAt(10, "B");
            PWM_draw_bar_timer_1();

            print_freq();

            _delay_ms(25);
    }

    return 0;
}

